@@ -34,7 +34,7 @@ enum {
 
 static guint signals [LAST_SIGNAL] = { 0, };
 
-G_DEFINE_TYPE (CsScreen, cs_screen, G_TYPE_OBJECT);
+G_DEFINE_TYPE (CsScreen, cs_screen, G_TYPE_OBJECT)
 
 static gboolean debug_mode = FALSE;
 #define DEBUG(...) if (debug_mode) g_printerr (__VA_ARGS__)
@@ -414,22 +414,18 @@ reload_screen_info (CsScreen *screen)
 static void
 on_monitors_changed (GdkScreen *gdk_screen, gpointer user_data)
 {
-    CsMonitorInfo *old_monitor_infos;
     CsScreen *screen;
+    CsMonitorInfo *old_monitor_infos;
 
     screen = CS_SCREEN (user_data);
 
-    reload_screen_info (screen);
-    g_signal_emit (screen, signals[SCREEN_SIZE_CHANGED], 0);
-
+    DEBUG ("CsScreen received 'monitors-changed' signal from GdkScreen %ld\n", g_get_monotonic_time () / 1000);
     gdk_flush ();
 
-    DEBUG ("CsScreen received 'monitors-changed' signal from GdkScreen\n");
-
     old_monitor_infos = screen->monitor_infos;
     reload_monitor_infos (screen);
-
     g_free (old_monitor_infos);
+    reload_screen_info (screen);
 
     g_signal_emit (screen, signals[SCREEN_MONITORS_CHANGED], 0);
 }
@@ -438,12 +434,18 @@ static void
 on_screen_changed (GdkScreen *gdk_screen, gpointer user_data)
 {
     CsScreen *screen;
+    CsMonitorInfo *old_monitor_infos;
 
     screen = CS_SCREEN (user_data);
 
-    DEBUG ("CsScreen received 'size-changed' signal from GdkScreen\n");
+    DEBUG ("CsScreen received 'size-changed' signal from GdkScreen %ld\n", g_get_monotonic_time () / 1000);
+    gdk_flush ();
 
+    old_monitor_infos = screen->monitor_infos;
+    reload_monitor_infos (screen);
+    g_free (old_monitor_infos);
     reload_screen_info (screen);
+
     g_signal_emit (screen, signals[SCREEN_SIZE_CHANGED], 0);
 }
 
@@ -735,8 +737,6 @@ cs_screen_get_low_res_mode (CsScreen *screen)
  *
  * Gets whether or not one of our monitors falls below the low res threshold (1200 wide).
  * This lets us display certain things at smaller sizes to prevent truncating of images, etc.
- *
- * Returns: Whether or not to use low res mode.
  */
 void
 cs_screen_get_smallest_monitor_sizes (CsScreen *screen,
@@ -801,19 +801,3 @@ cs_screen_reset_screensaver (void)
 {
     XResetScreenSaver (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
 }
-
-void
-cs_screen_nuke_focus (void)
-{
-    Window focus = 0;
-    int    rev = 0;
-
-    DEBUG ("Nuking focus\n");
-
-    gdk_error_trap_push ();
-
-    XGetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), &focus, &rev);
-    XSetInputFocus (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()), PointerRoot, RevertToNone, CurrentTime);
-
-    gdk_error_trap_pop_ignored ();
-}
